package com.digitalsanctuary.atg.rss;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

import javax.servlet.ServletException;

import atg.core.util.StringUtils;
import atg.nucleus.naming.ParameterName;
import atg.repository.Query;
import atg.repository.QueryBuilder;
import atg.repository.QueryExpression;
import atg.repository.QueryOptions;
import atg.repository.Repository;
import atg.repository.RepositoryException;
import atg.repository.RepositoryItem;
import atg.repository.RepositoryView;
import atg.repository.SortDirective;
import atg.repository.SortDirectives;
import atg.servlet.DynamoHttpServletRequest;
import atg.servlet.DynamoHttpServletResponse;
import atg.servlet.DynamoServlet;

/**
 * This droplet outputs an RSS 2.0 compliant feed based on Repository Items and configured property names.
 * 
 * @author Devon Hillard
 * 
 */
public class FeedDroplet extends DynamoServlet {

    /**
     * The output oparam.
     */
    private static final ParameterName OUTPUT = ParameterName.getParameterName("output");

    /**
     * the xml data param name.
     */
    private static final String DATA = "data";

    private atg.service.dynamo.Configuration mConfiguration;

    /**
     * The ATG Repository the items will come out of.
     */
    private Repository mRepository;

    /**
     * The maximum number of items to return.
     */
    private int mMaxItems;

    /**
     * The link template to use to construct links if the items do not have link properties.
     */
    private String mItemLink;

    /**
     * The item descriptor name of the repository item that will equate to a feed item.
     */
    private String mFeedItemItemDescriptor;

    /**
     * The property name of the repository item property to pull the feed item title from.
     */
    private String mFeedItemTitleProperty;

    /**
     * The property name of the repository item property to pull the feed item link from.
     */
    private String mFeedItemLinkProperty;

    /**
     * The property name of the repository item property to pull the feed item description from.
     */
    private String mFeedItemDescriptionProperty;

    /**
     * The property name of the repository item property to pull the feed item author from.
     */
    private String mFeedItemAuthorProperty;

    /**
     * The property name of the repository item property to pull the feed item category from.
     */
    private String mFeedItemCategoryProperty;

    /**
     * The property name of the repository item property to pull the feed item pub date from.
     */
    private String mFeedItemPubDateProperty;

    /**
     * The property name of the repository item property to pull the active flag from.
     */
    private String mFeedItemActiveProperty;

    /**
     * The value of an "active" flag from the active flag property of the repository item.
     */
    private String mFeedItemActiveFlagValue;

    // Required elements.
    /**
     * The name of the channel. It's how people refer to your service. If you have an HTML website that contains the
     * same information as your RSS file, the title of your channel should be the same as the title of your website.
     */
    private String mTitle;

    /**
     * The URL to the HTML website corresponding to the channel.
     */
    private String mLink;

    /**
     * Phrase or sentence describing the channel.
     */
    private String mDescription;
    // End Required elements.

    // Optional Elements.
    /**
     * The language the channel is written in. This allows aggregators to group all Italian language sites, for example,
     * on a single page. A list of allowable values for this element, as provided by Netscape, is here
     * (http://www.rssboard.org/rss-language-codes). You may also use values defined by the W3C
     * (http://www.w3.org/TR/REC-html40/struct/dirlang.html#langcodes).
     */
    private String mLanguage;

    /**
     * Copyright notice for content in the channel.
     */
    private String mCopyright;

    /**
     * Email address for person responsible for editorial content.
     */
    private String mManagingEditor;

    /**
     * Email address for person responsible for technical issues relating to channel.
     */
    private String mWebMaster;

    /**
     * The publication date for the content in the channel. For example, the New York Times publishes on a daily basis,
     * the publication date flips once every 24 hours. That's when the pubDate of the channel changes. All date-times in
     * RSS conform to the Date and Time Specification of RFC 822, with the exception that the year may be expressed with
     * two characters or four characters (four preferred).
     */
    private String mPubDate;

    /**
     * The last time the content of the channel changed.
     */
    private String mLastBuildDate;

    /**
     * Specify one or more categories that the channel belongs to. Follows the same rules as the <item>-level category
     * element. More info.
     */
    private String mCategory;

    /**
     * A string indicating the program used to generate the channel.
     */
    private String mGenerator;

    /**
     * A URL that points to the documentation for the format used in the RSS file. It's probably a pointer to this page.
     * It's for people who might stumble across an RSS file on a Web server 25 years from now and wonder what it is.
     */
    private String mDocs;

    /**
     * ttl stands for time to live. It's a number of minutes that indicates how long a channel can be cached before
     * refreshing from the source. More info here.
     */
    private String mTTL;

    // A channel image is options, however if you wish to have one, you must include the following three required image
    // related elements.
    // Required Image elements.
    /**
     * the URL of a GIF, JPEG or PNG image that represents the channel.
     */
    private String mImageURL;

    // Note: if the image URL is populated, but the title and link are not, they will default to the title and link of
    // the main channel.
    /**
     * describes the image, it's used in the ALT attribute of the HTML <img> tag when the channel is rendered in HTML.
     */
    private String mImageTitle;

    /**
     * the URL of the site, when the channel is rendered, the image is a link to the site. (Note, in practice the image
     * <title> and <link> should have the same value as the channel's <title> and <link>.
     */
    private String mImageLink;
    // End Required Image elements.

    // Optional Image elements.
    /**
     * Width of the image in pixels. Max value is 144, default is 88.
     */
    private String mImageWidth;

    /**
     * Height of the image in pixels. Max value is 400, default is 31.
     */
    private String mImageHeight;

    /**
     * contains text that is included in the TITLE attribute of the link formed around the image in the HTML rendering.
     */
    private String mImageDescription;
    // End Optional Image Elements.

    /**
     * The PICS rating for the channel.
     */
    private String mRating;

    // End Optional elements.

    /**
     * @see atg.servlet.DynamoServlet#service(atg.servlet.DynamoHttpServletRequest,
     *      atg.servlet.DynamoHttpServletResponse)
     */
    public void service(DynamoHttpServletRequest pRequest, DynamoHttpServletResponse pResponse)
	    throws ServletException, IOException {
	// Generate the XML
	StringBuffer XMLBuffer = new StringBuffer();
	// Create the header and channel xml
	XMLBuffer.append(createHeader());
	XMLBuffer.append(createChannel());
	// Create the Item XMLs
	XMLBuffer.append(createItems());
	// Create the footer XML
	XMLBuffer.append(createFooter());
	// Return the XML
	pRequest.setParameter(DATA, XMLBuffer.toString());
	pRequest.serviceLocalParameter(OUTPUT, pRequest, pResponse);
    }

    /**
     * Creates the header tags for the rss feed XML.
     * 
     * @return the xml for the header tags.
     */
    private String createHeader() {
	StringBuffer headerBuffer = new StringBuffer();
	headerBuffer.append("<?xml version=\"1.0\"?>");
	headerBuffer.append("<rss version=\"2.0\">");
	headerBuffer.append("<channel>");
	return headerBuffer.toString();
    }

    /**
     * Creates the footer tags for the rss feed XML.
     * 
     * @return the xml for the footer tags.
     */
    private String createFooter() {
	StringBuffer footerBuffer = new StringBuffer();
	footerBuffer.append("</channel>");
	footerBuffer.append("</rss>");
	return footerBuffer.toString();
    }

    /**
     * Creates the channel elements for the rss feed XML.
     * 
     * @return the xml for the channel elements.
     */
    private String createChannel() {
	StringBuffer channelBuffer = new StringBuffer();
	channelBuffer.append("<title>" + getTitle() + "</title>");
	channelBuffer.append("<link>" + getLink() + "</link>");
	channelBuffer.append("<description>" + getDescription() + "</description>");

	if (!StringUtils.isBlank(getLanguage())) {
	    channelBuffer.append("<language>" + getLanguage() + "</language>");
	}
	if (!StringUtils.isBlank(getCopyright())) {
	    channelBuffer.append("<copyright>" + getCopyright() + "</copyright>");
	}
	if (!StringUtils.isBlank(getCategory())) {
	    channelBuffer.append("<category>" + getCategory() + "</category>");
	}
	if (!StringUtils.isBlank(getPubDate())) {
	    channelBuffer.append("<pubDate>" + getPubDate() + "</pubDate>");
	}
	if (!StringUtils.isBlank(getLastBuildDate())) {
	    channelBuffer.append("<lastBuildDate>" + getLastBuildDate() + "</lastBuildDate>");
	}
	if (!StringUtils.isBlank(getDocs())) {
	    channelBuffer.append("<docs>" + getDocs() + "</docs>");
	}
	if (!StringUtils.isBlank(getGenerator())) {
	    channelBuffer.append("<generator>" + getGenerator() + "</generator>");
	}
	if (!StringUtils.isBlank(getManagingEditor())) {
	    channelBuffer.append("<managingEditor>" + getManagingEditor() + "</managingEditor>");
	}
	if (!StringUtils.isBlank(getWebMaster())) {
	    channelBuffer.append("<webMaster>" + getWebMaster() + "</webMaster>");
	}
	if (!StringUtils.isBlank(getTTL())) {
	    channelBuffer.append("<ttl>" + getTTL() + "</ttl>");
	}
	if (!StringUtils.isBlank(getRating())) {
	    channelBuffer.append("<rating>" + getRating() + "</rating>");
	}

	if (!StringUtils.isBlank(getImageURL())) {
	    channelBuffer.append("<image>");
	    channelBuffer.append("<url>" + getImageURL() + "</url>");
	    if (StringUtils.isBlank(getImageTitle())) {
		channelBuffer.append("<title>" + getTitle() + "</title>");
	    } else {
		channelBuffer.append("<title>" + getImageTitle() + "</title>");
	    }
	    if (StringUtils.isBlank(getImageLink())) {
		channelBuffer.append("<link>" + getLink() + "</link>");
	    } else {
		channelBuffer.append("<link>" + getImageLink() + "</link>");
	    }
	    if (!StringUtils.isBlank(getImageWidth())) {
		channelBuffer.append("<width>" + getImageWidth() + "</width>");
	    }
	    if (!StringUtils.isBlank(getImageHeight())) {
		channelBuffer.append("<height>" + getImageHeight() + "</height>");
	    }
	    if (!StringUtils.isBlank(getImageDescription())) {
		channelBuffer.append("<description>" + getImageDescription() + "</description>");
	    }
	    channelBuffer.append("</image>");
	}
	return channelBuffer.toString();
    }

    /**
     * This method iterates through the loaded RepositoryItems and creates the xml for them. This method also breaks
     * down the itemLink property, if it is populated, into three components, firstPart, propertyName, and lastPart.
     * 
     * @return the xml for all of the RepositoryItems.
     */
    private String createItems() {
	StringBuffer itemsBuffer = new StringBuffer();
	// Load the items
	RepositoryItem[] feedItemArray = loadItems();
	List feedItemList = Arrays.asList(feedItemArray);
	Iterator feedItemListIter = feedItemList.iterator();

	// Setup itemLink based variables. Most items may not have a usable link property, so we expect to have to
	// construct a link based on the ItemLink property.
	String firstPartOfLink = null;
	String lastPartOfLink = null;
	String linkPropertyName = null;
	if (!StringUtils.isBlank(getItemLink())) {
	    int indexOfStart = getItemLink().indexOf("#{");
	    if (indexOfStart < 0) {
		indexOfStart = getItemLink().length();
	    }
	    if (getItemLink().startsWith("http")) {
		firstPartOfLink = getItemLink().substring(0, indexOfStart);
	    } else {
		firstPartOfLink = "http://" + getConfiguration().getSiteHttpServerName() + ":"
			+ getConfiguration().getSiteHttpServerPort() + getItemLink().substring(0, indexOfStart);
	    }
	    if (indexOfStart < getItemLink().length()) {
		int indexOfEnd = getItemLink().indexOf("}", indexOfStart);

		linkPropertyName = getItemLink().substring(indexOfStart + 2, indexOfEnd);
		if (indexOfEnd < getItemLink().length()) {
		    lastPartOfLink = getItemLink().substring(indexOfEnd + 1, getItemLink().length());
		}
	    }
	}
	// Iterate through all of the RepositoryItems
	while (feedItemListIter.hasNext()) {
	    RepositoryItem item = (RepositoryItem) feedItemListIter.next();
	    if (item != null) {
		itemsBuffer.append("<item>");
		if (!StringUtils.isBlank(getFeedItemTitleProperty())
			&& !StringUtils.isBlank(getStringProperty(item, getFeedItemTitleProperty()))) {
		    itemsBuffer.append("<title>" + getStringProperty(item, getFeedItemTitleProperty()) + "</title>");
		}
		// If the repository Item has a link property use it
		if (!StringUtils.isBlank(getFeedItemLinkProperty())
			&& !StringUtils.isBlank(getStringProperty(item, getFeedItemLinkProperty()))) {
		    itemsBuffer.append("<link>" + getStringProperty(item, getFeedItemLinkProperty()) + "</link>");
		} else if (!StringUtils.isBlank(firstPartOfLink)) {
		    // Otherwise build one based on the link we parsed above.
		    itemsBuffer.append("<link>");
		    itemsBuffer.append(firstPartOfLink);
		    if (!StringUtils.isBlank(linkPropertyName)) {
			String linkPropertyValue = null;
			// Handle the special case for repositoryId
			if (linkPropertyName.equals("repositoryId")) {
			    linkPropertyValue = item.getRepositoryId();
			} else {
			    linkPropertyValue = getStringProperty(item, linkPropertyName);
			}
			if (!StringUtils.isBlank(linkPropertyValue)) {
			    itemsBuffer.append(linkPropertyValue);
			}
		    }
		    if (!StringUtils.isBlank(lastPartOfLink)) {
			itemsBuffer.append(lastPartOfLink);
		    }
		    itemsBuffer.append("</link>");
		}
		if (!StringUtils.isBlank(getFeedItemDescriptionProperty())
			&& !StringUtils.isBlank(getStringProperty(item, getFeedItemDescriptionProperty()))) {
		    itemsBuffer.append("<description>" + getStringProperty(item, getFeedItemDescriptionProperty())
			    + "</description>");
		}

		if (!StringUtils.isBlank(getFeedItemAuthorProperty())
			&& !StringUtils.isBlank(getStringProperty(item, getFeedItemAuthorProperty()))) {
		    itemsBuffer.append("<author>" + getStringProperty(item, getFeedItemAuthorProperty()) + "</author>");
		}
		if (!StringUtils.isBlank(getFeedItemCategoryProperty())
			&& !StringUtils.isBlank(getStringProperty(item, getFeedItemCategoryProperty()))) {
		    itemsBuffer.append("<category>" + getStringProperty(item, getFeedItemCategoryProperty())
			    + "</category>");
		}
		if (!StringUtils.isBlank(getFeedItemPubDateProperty())
			&& !StringUtils.isBlank(getStringProperty(item, getFeedItemPubDateProperty()))) {
		    itemsBuffer.append("<pubDate>" + getStringProperty(item, getFeedItemPubDateProperty())
			    + "</pubDate>");
		}
		// Use the repositoryId as the non-link GUID.
		itemsBuffer.append("<guid isPermaLink=\"false\">" + item.getRepositoryId() + "</guid>");
		itemsBuffer.append("</item>");
	    }
	}
	return itemsBuffer.toString();
    }

    /**
     * This method loads up an array of RepositoryItems, sorted by the PubDate property, and filtered on the active flag
     * if it is provided.
     * 
     * @return an array of RepositoryItems.
     */
    private RepositoryItem[] loadItems() {
	RepositoryItem[] feedItemArray = null;
	try {
	    RepositoryView view = getRepository().getView(getFeedItemItemDescriptor());
	    Query query = null;
	    if (StringUtils.isBlank(getFeedItemActiveProperty())) {
		query = view.getQueryBuilder().createUnconstrainedQuery();
	    } else {
		QueryBuilder queryBuilder = view.getQueryBuilder();
		QueryExpression activeFlag = queryBuilder.createPropertyQueryExpression(getFeedItemActiveProperty());
		QueryExpression activeValue = queryBuilder.createConstantQueryExpression(getFeedItemActiveFlagValue());
		query = view.getQueryBuilder().createComparisonQuery(activeFlag, activeValue, QueryBuilder.EQUALS);
	    }

	    String[] precachedPropertyNames = buildPropertyNameArray();
	    SortDirectives sortDirectives = new SortDirectives();
	    sortDirectives.addDirective(new SortDirective(getFeedItemPubDateProperty(), SortDirective.DIR_DESCENDING));
	    feedItemArray = view.executeQuery(query, new QueryOptions(0, getMaxItems(), sortDirectives,
		    precachedPropertyNames));

	} catch (RepositoryException e) {
	    logError(e);
	}
	logError("loaded items: " + feedItemArray.length);
	return feedItemArray;
    }

    /**
     * This method looks for populated property names to use in generating the resultant XML feed and compiles them into
     * an array to drive the precaching of the query.
     * 
     * @return an array of property names to precache, as they will be used to construct the xml feed.
     */
    private String[] buildPropertyNameArray() {
	List propertyNameList = new ArrayList();
	if (!StringUtils.isBlank(getFeedItemAuthorProperty())) {
	    propertyNameList.add(getFeedItemAuthorProperty());
	}
	if (!StringUtils.isBlank(getFeedItemCategoryProperty())) {
	    propertyNameList.add(getFeedItemCategoryProperty());
	}
	if (!StringUtils.isBlank(getFeedItemDescriptionProperty())) {
	    propertyNameList.add(getFeedItemDescriptionProperty());
	}
	if (!StringUtils.isBlank(getFeedItemLinkProperty())) {
	    propertyNameList.add(getFeedItemLinkProperty());
	}
	if (!StringUtils.isBlank(getFeedItemPubDateProperty())) {
	    propertyNameList.add(getFeedItemPubDateProperty());
	}
	if (!StringUtils.isBlank(getFeedItemTitleProperty())) {
	    propertyNameList.add(getFeedItemTitleProperty());
	}
	return (String[]) propertyNameList.toArray(new String[0]);
    }

    /**
     * This method returns the property.toString() representation, and also traverses nested properties using the "."
     * syntax.
     * 
     * @param pItem
     *                the RepositoryItem to look in.
     * @param pPropertyName
     *                the property name (or nested name).
     * @return the toString() representation of the property.
     */
    private String getStringProperty(RepositoryItem pItem, String pPropertyName) {
	if (pItem == null || pPropertyName == null) {
	    return null;
	}
	// Is this a nested property?
	if (pPropertyName.indexOf(".") > 0) {
	    String firstItemName = pPropertyName.substring(0, pPropertyName.indexOf("."));
	    return getStringProperty((RepositoryItem) pItem.getPropertyValue(firstItemName), pPropertyName
		    .substring(pPropertyName.indexOf(".") + 1));
	} else {
	    Object value = pItem.getPropertyValue(pPropertyName);
	    if (value != null) {
		return pItem.getPropertyValue(pPropertyName).toString();
	    } else {
		return null;
	    }
	}
    }

    public String getTitle() {
	return mTitle;
    }

    public void setTitle(String pTitle) {
	mTitle = pTitle;
    }

    public String getLink() {
	return mLink;
    }

    public void setLink(String pLink) {
	mLink = pLink;
    }

    public String getDescription() {
	return mDescription;
    }

    public void setDescription(String pDescription) {
	mDescription = pDescription;
    }

    public String getLanguage() {
	return mLanguage;
    }

    public void setLanguage(String pLanguage) {
	mLanguage = pLanguage;
    }

    public String getCopyright() {
	return mCopyright;
    }

    public void setCopyright(String pCopyright) {
	mCopyright = pCopyright;
    }

    public String getManagingEditor() {
	return mManagingEditor;
    }

    public void setManagingEditor(String pManagingEditor) {
	mManagingEditor = pManagingEditor;
    }

    public String getWebMaster() {
	return mWebMaster;
    }

    public void setWebMaster(String pWebMaster) {
	mWebMaster = pWebMaster;
    }

    public String getPubDate() {
	return mPubDate;
    }

    public void setPubDate(String pPubDate) {
	mPubDate = pPubDate;
    }

    public String getLastBuildDate() {
	return mLastBuildDate;
    }

    public void setLastBuildDate(String pLastBuildDate) {
	mLastBuildDate = pLastBuildDate;
    }

    public String getCategory() {
	return mCategory;
    }

    public void setCategory(String pCategory) {
	mCategory = pCategory;
    }

    public String getGenerator() {
	return mGenerator;
    }

    public void setGenerator(String pGenerator) {
	mGenerator = pGenerator;
    }

    public String getDocs() {
	return mDocs;
    }

    public void setDocs(String pDocs) {
	mDocs = pDocs;
    }

    public String getTTL() {
	return mTTL;
    }

    public void setTTL(String pTtl) {
	mTTL = pTtl;
    }

    public String getImageURL() {
	return mImageURL;
    }

    public void setImageURL(String pImageURL) {
	mImageURL = pImageURL;
    }

    public String getImageTitle() {
	return mImageTitle;
    }

    public void setImageTitle(String pImageTitle) {
	mImageTitle = pImageTitle;
    }

    public String getImageLink() {
	return mImageLink;
    }

    public void setImageLink(String pImageLink) {
	mImageLink = pImageLink;
    }

    public String getImageWidth() {
	return mImageWidth;
    }

    public void setImageWidth(String pImageWidth) {
	mImageWidth = pImageWidth;
    }

    public String getImageHeight() {
	return mImageHeight;
    }

    public void setImageHeight(String pImageHeight) {
	mImageHeight = pImageHeight;
    }

    public String getImageDescription() {
	return mImageDescription;
    }

    public void setImageDescription(String pImageDescription) {
	mImageDescription = pImageDescription;
    }

    public String getRating() {
	return mRating;
    }

    public void setRating(String pRating) {
	mRating = pRating;
    }

    public Repository getRepository() {
	return mRepository;
    }

    public void setRepository(Repository pRepository) {
	mRepository = pRepository;
    }

    public int getMaxItems() {
	return mMaxItems;
    }

    public void setMaxItems(int pMaxItems) {
	mMaxItems = pMaxItems;
    }

    public String getFeedItemItemDescriptor() {
	return mFeedItemItemDescriptor;
    }

    public void setFeedItemItemDescriptor(String pFeedItemItemDescriptor) {
	mFeedItemItemDescriptor = pFeedItemItemDescriptor;
    }

    public String getFeedItemTitleProperty() {
	return mFeedItemTitleProperty;
    }

    public void setFeedItemTitleProperty(String pFeedItemTitleProperty) {
	mFeedItemTitleProperty = pFeedItemTitleProperty;
    }

    public String getFeedItemLinkProperty() {
	return mFeedItemLinkProperty;
    }

    public void setFeedItemLinkProperty(String pFeedItemLinkProperty) {
	mFeedItemLinkProperty = pFeedItemLinkProperty;
    }

    public String getFeedItemDescriptionProperty() {
	return mFeedItemDescriptionProperty;
    }

    public void setFeedItemDescriptionProperty(String pFeedItemDescriptionProperty) {
	mFeedItemDescriptionProperty = pFeedItemDescriptionProperty;
    }

    public String getFeedItemAuthorProperty() {
	return mFeedItemAuthorProperty;
    }

    public void setFeedItemAuthorProperty(String pFeedItemAuthorProperty) {
	mFeedItemAuthorProperty = pFeedItemAuthorProperty;
    }

    public String getFeedItemCategoryProperty() {
	return mFeedItemCategoryProperty;
    }

    public void setFeedItemCategoryProperty(String pFeedItemCategoryProperty) {
	mFeedItemCategoryProperty = pFeedItemCategoryProperty;
    }

    public String getFeedItemPubDateProperty() {
	return mFeedItemPubDateProperty;
    }

    public void setFeedItemPubDateProperty(String pFeedItemPubDateProperty) {
	mFeedItemPubDateProperty = pFeedItemPubDateProperty;
    }

    /**
     * @return the feedItemActiveProperty
     */
    public String getFeedItemActiveProperty() {
	return mFeedItemActiveProperty;
    }

    /**
     * @param pFeedItemActiveProperty
     *                the feedItemActiveProperty to set
     */
    public void setFeedItemActiveProperty(String pFeedItemActiveProperty) {
	mFeedItemActiveProperty = pFeedItemActiveProperty;
    }

    /**
     * @return the feedItemActiveFlagValue
     */
    public String getFeedItemActiveFlagValue() {
	return mFeedItemActiveFlagValue;
    }

    /**
     * @param pFeedItemActiveFlagValue
     *                the feedItemActiveFlagValue to set
     */
    public void setFeedItemActiveFlagValue(String pFeedItemActiveFlagValue) {
	mFeedItemActiveFlagValue = pFeedItemActiveFlagValue;
    }

    /**
     * @return the itemLink
     */
    public String getItemLink() {
	return mItemLink;
    }

    /**
     * @param pItemLink
     *                the itemLink to set
     */
    public void setItemLink(String pItemLink) {
	mItemLink = pItemLink;
    }

    /**
     * @return the configuration
     */
    public atg.service.dynamo.Configuration getConfiguration() {
	return mConfiguration;
    }

    /**
     * @param pConfiguration
     *                the configuration to set
     */
    public void setConfiguration(atg.service.dynamo.Configuration pConfiguration) {
	mConfiguration = pConfiguration;
    }
}
